/**
* HipChat plugin for the uptime project - https://github.com/fzaninotto/uptime
*
* This index.js files goes to a directory `plugins/hipchat` in your installation of uptime.
*
* Notifies all events (up, down, paused, restarted) to HipChat
*
* This plugin has a dependency on `hipchatter`.
* Add this to the "dependencies" object in your `package.json` file :
*
*   "hipchatter":    "0.2.0"
*
* To enable the plugin, add the following line to the plugins section of your config file
* plugins:
*  - ./plugins/hipchat
*
* Example configuration
*
*
*   hipchat:
*     roomId: 1  # HipChat API Room ID or name (see Hipchatter docs)
*     token: your-hipchat-room-notification-token
*     uptimeDashboardURL: https://your-uptime-installation.example.com
*
*     event:
*       up:        true
*       down:      true
*       paused:    false
*       restarted: false
*/

var fs   = require('fs');
var ejs  = require('ejs');
var template = fs.readFileSync(__dirname + '/views/_detailsEdit.ejs', 'utf8');

var config     = require('config').hipchat;
var CheckEvent = require('../../models/checkEvent');
var Hipchatter = require('hipchatter');

var roomId = config.roomId;
var token = config.token;
var uptimeDashboardURL = config.uptimeDashboardURL;

exports.initWebApp = function(options) {

  var link = function(text, url) {
    return '<a href="' + url + '">' + text + '</a>';
  };

  var image = function(alt, url, width, height) {
    var size = '';
    if (width != undefined) {
      size += ' width="' + width + 'px"';
    }
    if (height != undefined) {
      size += ' height="' + height + 'px"';
    }
    else {
      size += ' height="32px"';
    }
    return '<img' + size + ' src="' + url + '" />';
  };

  var getToken = function(check) {
    if (check.pollerParams !== undefined && check.pollerParams.hipchat_token !== undefined && check.pollerParams.hipchat_token !== '') {
      return check.pollerParams.hipchat_token;
    }
    return token;
  };

  var getRoomId = function(check) {
    if (check.pollerParams !== undefined && check.pollerParams.hipchat_roomid !== undefined && check.pollerParams.hipchat_roomid !== '') {
      return check.pollerParams.hipchat_roomid;
    }
    return roomId;
  };

  var sendMessage = function(check, text) {
    var checkToken = getToken(check);
    var checkRoomId = getRoomId(check);
    var hipchatter = new Hipchatter(checkToken);
    var payload = {
      'token': checkToken,
      'color': 'purple',
      'message': text,
      'notify': true,
      'message_format': 'html'
    };

    hipchatter.notify(getRoomId(check), payload, function(err) {
        if (err == null) console.log('HipChat notification successful. Sent: ' + payload);
      }
    );
  };

  CheckEvent.on('afterInsert', function(checkEvent) {

    if (!config.event[checkEvent.message]) {
      console.log('Event ' + checkEvent.message + ' is not enabled in the event section of HipChat config. Will not notify.');
      return;
    } else {
      console.log('Event ' + checkEvent.message + 'caught. Will notify!');

    }

    checkEvent.findCheck(function(err, check) {
      if (err) {
        console.log('HipChat Plugin Error: ' + err);
      }

      sendMessage(check, 'Application ' + link(check.name, uptimeDashboardURL + '/dashboard/checks/' + check._id) + ' -> ' + checkEvent.message);
    });
  });

  var api = options.api;
  var dashboard = options.dashboard;

  api.on('populateFromDirtyCheck', function(checkDocument, dirtyCheck, type) {
    dashboard.emit('populateFromDirtyCheck', checkDocument, dirtyCheck, type);
  });

  dashboard.on('populateFromDirtyCheck', function(checkDocument, dirtyCheck, type) {
    if (!dirtyCheck.hipchat_token || dirtyCheck.hipchat_token == '') {
      dirtyToken = '';
    }
    else {
      dirtyToken = dirtyCheck.hipchat_token;
    }
    if (!dirtyCheck.hipchat_roomid || dirtyCheck.hipchat_roomid == '') {
      dirtyRoomId = '';
    }
    else {
      dirtyRoomId = dirtyCheck.hipchat_roomid;
    }
    checkDocument.setPollerParam('hipchat_token', dirtyToken);
    checkDocument.setPollerParam('hipchat_roomid', dirtyRoomId);
  });

  dashboard.on('checkEdit', function(type, check, partial) {
    partial.push(ejs.render(template, { locals: { check: check } }));
  });

  dashboard.on('newTrackerItem', function(source, item) {
    sendMessage(source.check, source.renderItem(item, link, image));
  });

  console.log('Enabled HipChat notifications');

};
