/**
 * Mattermost plugin for the uptime project - https://github.com/fzaninotto/uptime
 *
 * This index.js files goes to a directory `plugins/mattermost` in your installation of uptime.
 *
 * Notifies all events (up, down, paused, restarted) to Mattermost
 *
 * This plugin has a dependency on `node-mattermost`.
 * Add this to the "dependencies" object in your `package.json` file :
 *
 *   "node-mattermost": "^0.0.1"
 *
 * To enable the plugin, add the following line to the plugins section of your config file
 * plugins:
 *  - ./plugins/mattermost
 *
 * Example configuration
 *
 *   mattermost:
 *     hook: https://mattermost.example.com/hooks/<token>
 *     channel: #devop
 *     username: Bot
 *     gitlab: True
 *     uptimeDashboardURL: https://your-uptime-installation.example.com
 *
 *     event:
 *       up:        true
 *       down:      true
 *       paused:    false
 *       restarted: false
 */

var fs   = require('fs');
var ejs  = require('ejs');
var template = fs.readFileSync(__dirname + '/views/_detailsEdit.ejs', 'utf8');

var config     = require('config').mattermost;
var CheckEvent = require('../../models/checkEvent');
var Mattermost = require('node-mattermost');

var hook = config.hook;
var channel = config.channel || false;
var username = config.username || false;
var gitlab = config.gitlab || false;
var uptimeDashboardURL = config.uptimeDashboardURL;

var symbol = {
  up: ':white_check_mark:',
  down: ':warning:',
  paused: ':warning:',
  restarted: ':white_check_mark:'
};

exports.initWebApp = function(options) {

  var link = function(text, url) {
    return '[' + text + '](' + url  + ')';
  };

  var image = function(alt, url, width, height) {
      var size = '';
      if (width != undefined) {
          size = ' =' + width + 'x';
          if (height != undefined) {
              size += height;
          }
      }
      return '![' + alt + '](' + url + size + ')';
  };

  var getHook = function(check) {
    if (check.pollerParams !== undefined && check.pollerParams.mattermost_hook !== undefined && check.pollerParams.mattermost_hook !== '') {
      return check.pollerParams.mattermost_hook;
    }
    return hook;
  };

  var sendMessage = function(check, text) {
    var mattermost = new Mattermost(getHook(check));
    var payload = {
      text: text
    };
    if (!gitlab) {
      payload['channel'] = channel;
    }
    if (username) {
      payload['username'] = username;
    }

    mattermost.send(payload, function(err) {
      if (err == null) console.log('Mattermost notification successful. Sent: ' + payload);
    });
  };

  CheckEvent.on('afterInsert', function(checkEvent) {

    if (!config.event[checkEvent.message]) {
      console.log('Event ' + checkEvent.message + ' is not enabled in the event section of Mattermost config. Will not notify.');
      return;
    } else {
      console.log('Event ' + checkEvent.message + ' caught. Will notify!');
    }

    checkEvent.findCheck(function(err, check) {
      if (err) {
        console.log('Mattermost Plugin Error: ' + err);
      }
      sendMessage(check, 'Application ' + link(check.name, uptimeDashboardURL + '/dashboard/checks/' + check._id) + ' -> ' + checkEvent.message + ' ' + symbol[checkEvent.message]);
    });
  });

  var api = options.api;
  var dashboard = options.dashboard;

  api.on('populateFromDirtyCheck', function(checkDocument, dirtyCheck, type) {
    dashboard.emit('populateFromDirtyCheck', checkDocument, dirtyCheck, type);
  });

  dashboard.on('populateFromDirtyCheck', function(checkDocument, dirtyCheck, type) {
    if (!dirtyCheck.mattermost_hook || dirtyCheck.mattermost_hook == '') {
      checkDocument.setPollerParam('mattermost_hook', '');
      return;
    }
    checkDocument.setPollerParam('mattermost_hook', dirtyCheck.mattermost_hook);
  });

  dashboard.on('checkEdit', function(type, check, partial) {
    partial.push(ejs.render(template, { locals: { check: check } }));
  });

  dashboard.on('newTrackerItem', function(source, item) {
    sendMessage(source.check, source.renderItem(item, link, image));
  });

  console.log('Enabled Mattermost notifications');

};
