/**
 * Module dependencies.
 */
var GitHubApi= require('github');
var util = require('util');
var BaseTrackerSource = require('../baseSource');
var github;

/**
 * GitHub TrackerSource, to receive latest notifications and report to targets
 *
 * @param {String} id
 * @param {Integer} lastActivity
 * @param {String} url
 * @param {Array} params
 * @api   public
 */
function GitHubTrackerSource(id, lastActivity, url, params) {
  GitHubTrackerSource.super_.call(this, id, lastActivity, url, params);
}

util.inherits(GitHubTrackerSource, BaseTrackerSource);

GitHubTrackerSource.type = 'GitHub';

GitHubTrackerSource.prototype.poll = function() {
  var source = this;
  github = new GitHubApi( {
    host: source.url.substr(source.url.search('://') + 3)
  });
  github.authenticate({
    type: "token",
    token: source.password
  });
  github.activity.getNotifications({}, function(err, res) {
    if (err) {
      source.debug(err);
      return;
    }
    if (res === undefined) {
      return;
    }
    var issues = [];
    for(var key in res) {
      if (key != 'meta') {
        issues[key] = res[key];
      }
    }
    source.readItems(issues, false);
  });
};

GitHubTrackerSource.prototype.notifyItem = function(item) {
  github.activity.markNotificationThreadAsRead({
    id: item.id
  });
  return true;
};

GitHubTrackerSource.prototype.renderItem = function(item, linkCB, imageCB) {
  var url = item.repository.html_url,
    avatar = imageCB('', item.repository.owner.avatar_url, 50),
    project = item.repository.full_name,
    title = item.subject.title,
    type = '';

  switch(item.subject.type) {
    case 'Issue':
      var issue_id = item.subject.url.split('/').pop(),
        comment_id = item.subject.latest_comment_url.split('/').pop();
      url = url + '/issues/' + issue_id + '#issuecomment-' + comment_id;
      type = 'Issue';
      break;
    case 'PullRequest':
      var issue_id = item.subject.url.split('/').pop();
      url = url + '/pull/' + issue_id;
      type = 'PR';
      break;
    default:
      url = item.subject.url.replace('//api.', '//').replace('/repos/', '/');
      type = 'Unknown';
  }
  return avatar + ' ' + type + ' in ' + project + ': ' + linkCB(title, url);
};

GitHubTrackerSource.prototype.getItemDate = function(item) {
  return item.updated_at;
};

GitHubTrackerSource.prototype.compareDate = function(a, b) {
  return (Date.parse(this.getItemDate(a)) - Date.parse(this.getItemDate(b)));
};

GitHubTrackerSource.prototype.isItemNew = function(item) {
  return true;
};

GitHubTrackerSource.prototype.formatDate = function(timestamp) {
  return timestamp;
};

module.exports = GitHubTrackerSource;
