/**
 * Module dependencies.
 */
var request = require('request');
var util = require('util');
var BaseTrackerSource = require('../baseSource');

/**
 * YouTrack TrackerSource, to receive latest changes from and report to targets
 *
 * @param {String} id
 * @param {Integer} lastActivity
 * @param {String} url
 * @param {Array} params
 * @api   public
 */
function YouTrackTrackerSource(id, lastActivity, url, params) {
  YouTrackTrackerSource.super_.call(this, id, lastActivity, url, params);
}

util.inherits(YouTrackTrackerSource, BaseTrackerSource);

YouTrackTrackerSource.type = 'YouTrack';

YouTrackTrackerSource.strings = {
  recent: {
    en: '(for: me or commenter: me or reported by: me)',
    de: '(für: mich oder kommentiert von: mich oder ersteller: mich)'
  },
  project: {
    en: ' and project',
    de: ' und projekt'
  },
  unresolved: {
    en: ' and #unresolved',
    de: ' und #ungelöst'
  },
  updated: {
    en: ' and updated: ',
    de: ' und aktualisierungsdatum: '
  },
  order: {
    en: ' order by: updated asc',
    de: ' sortieren nach: aktualisierungsdatum aufsteigend'
  }
};

YouTrackTrackerSource.prototype.authenticate = function(callback) {
  var source = this;
  request
    .post({
      jar: true,
      url: source.url + '/rest/user/login',
      form: {
        login: source.username,
        password: source.password
      }
    }, function (err, res, body) {
      if (err || res.statusCode !== 200) {
        err = err || new Error('Wrong credentials.');
        source.debug(err);
        return;
      }
      callback();
    });
};

YouTrackTrackerSource.prototype.poll = function() {
  var source = this;
  this.authenticate(function() {
    var filter = source.t('recent');
    if (source.project_include) {
      source.project_include.split(',').forEach(function(prj) {
        filter += source.t('project') + ': ' + prj.trim();
      });
    }
    if (source.project_exclude) {
      source.project_exclude.split(',').forEach(function(prj) {
        filter += source.t('project') + ': -' + prj.trim();
      });
    }
    if (source.unresolvedOnly) {
      filter += source.t('unresolved');
    }
    if (source.lastActivity > 0) {
      filter += source.t('updated') + source.formatDate(source.lastActivity) + ' .. ' + source.formatDate();
    }
    filter += source.t('order');
    // TODO: We are receiving up to 10.000 items per request and we should break this into chunk and read multiple times
    var url = source.url + '/rest/issue?with=updated&with=summary&with=updaterFullName&with=updaterName&max=10000&filter=' + encodeURIComponent(filter);
    source.debug(url);
    request
      .get({
        jar: true,
        url: url,
        headers: {
          'Accept': 'application/json'
        }
      }, function(err, response, body) {
        if (err || response.statusCode !== 200){
          err = err || new Error('Unknown error.');
          source.debug(err);
          return;
        }

        var res = JSON.parse(body);
        source.debug(body);
        source.readItems(res.issue, false);
      })
      .on('error', function(e) {
        source.debug('ERROR: ' + e.message);
      });
  });
};

YouTrackTrackerSource.prototype.notifyItem = function(item) {
  return !(this.hideMine && (this.getValue(item.field, 'updaterName') == this.username));
};

YouTrackTrackerSource.prototype.renderItem = function(item, linkCB, imageCB) {
  return this.getValue(item.field, 'updaterFullName') + ': ' + linkCB(item.id, this.url + '/issue/' + item.id) + ' ' + this.getValue(item.field, 'summary');
};

YouTrackTrackerSource.prototype.getItemDate = function(item) {
  return this.getValue(item.field, 'updated');
};

YouTrackTrackerSource.prototype.formatDate = function(timestamp) {
  timestamp = timestamp || Date.now();
  date = new Date();
  date.setTime(parseInt(timestamp) + (parseInt(this.timeshift) * 3600000) + 1000);
  return date.toISOString().substr(0, 19);
};

YouTrackTrackerSource.prototype.getValue = function(values, key) {
  var output = '';
  values.forEach(function(value) {
    if (value['name'] == key) {
      output = value['value'];
    }
  });
  return output;
};

module.exports = YouTrackTrackerSource;
